#include "DancingCube.h"

/* initiate melody */
void IO_music_init(void) {
		melody = mario;
		current_duration = mario_size;
		melody_size = sizeof(mario) / sizeof(int);
}

/* initializations of pins */
void IO_init(void) {
	/* for cube */
	DDRA |= FULL;
	DDRC |= FULL;
	DDRB |= FULL;

	/* for transistors */
	DDRD |= (1 << PD0) | (1 << PD1) | (1 << PD4) | (1 << PD7);

}

/* light all the lights */
void light_cube(void)
{
	/* activate layers */
	PORTD |= (1<<LAYER1);
	PORTD |= (1<<LAYER2);
	PORTD |= (1<<LAYER3);
	PORTD |= (1<<LAYER4);

	/* activate pins */
	PORTA = FULL;
	PORTC = FULL;

}

/* turn off all lights from cube */
void turn_off_cube(void)
{
	/* disable layers */
	PORTD &= ~((1<<LAYER1));
	PORTD &= ~((1<<LAYER2));
	PORTD &= ~((1<<LAYER3));
	PORTD &= ~((1<<LAYER4));

	/* disable pins */
	PORTA = EMPTY;
	PORTC = EMPTY;
}

/* activate just one layer */
void layer_on(int layer)
{
	PORTD |= (1 << layers[layer]);

	PORTA = FULL;
	PORTC = FULL;
}

/* disable a layer */
void layer_off(int layer)
{
	PORTD &= ~(1 << layers[layer]);

	PORTA = EMPTY;
	PORTC = EMPTY;
}

/* turn on a line */
void line_on(int v)
{
	if (v >= 4)
		return;

	/* activate layers */
	PORTD |= (1 << layers[0]) | (1 << layers[1]) | (1 << layers[2]) | (1 << layers[3]);

	/* activate pins*/
	PORTC |= (1 << pins[v]) | (1 << pins[v + 4]);
	PORTA |= (1 << pins[v + 8]) | (1 << pins[v + 12]);

}

/* turn off a line */
void line_off(int v)
{

	if (v >= 4)
		return;

	/* disable layers */
	PORTD &= ~(1 << layers[0]) & ~(1 << layers[1])
				& ~(1 << layers[2]) & ~(1 << layers[3]);

	/* disable pins */
	PORTC &= ~(1 << pins[v]) & ~(1 << pins[v + 4]);
	PORTA &= ~(1 << pins[v + 8]) & ~ (1 << pins[v + 12]);

}
/* rain effect */
void drop(int p)
{
	int i;

	if (p < 8) {
		/* activate for port C pins */
		PORTC |= (1 << pins[p]);
		for (i = 0; i < 4; i++) {
			PORTD |= (1 << layers[i]);
			_delay_ms(100);
			PORTD &= ~ (1 << layers[i]);
		}
		PORTC &= ~(1 << pins[p]);
	} else {
		/* activate for port A pins */
		PORTA |= (1 << pins[p]);
		for (i = 0; i < 4; i++) {
			PORTD |= (1 << layers[i]);
			_delay_ms(150);
			PORTD &= ~ (1 << layers[i]);
		}
		PORTA &= ~(1 << pins[p]);
	}
}

void rain_effect(int iterations)
{
	int i, n;
	for (i = 0; i < iterations; i++) {
		/* random n */
		n = (int)((double)rand() / ((double)RAND_MAX + 1) * 16);
		drop(n);
	}
}


/* turn on the inner cube */
void turn_on_in_cube(void)
{
	PORTA |= (1 << PA2) | (1 << PA1);
	PORTC |= (1 << PC5) | (1 << PC6);
	PORTD |= (1 << PD1) | (1 << PD4);
}

/* turn on the out cube */
void turn_on_out_cube(void)
{
	PORTA |= (1 << PA6) | (1 << PA5);
	PORTC |= (1 << PC1) | (1 << PC2);

	line_on(0);
	line_on(3);
}

/* turn off the inner cube */
void turn_off_in_cube(void) {
	PORTA &= ~((1 << PA2) | (1 << PA1));
	PORTC &= ~((1 << PC5) | (1 << PC6));
	PORTD &= ~((1 << PD1) | (1 << PD4));
}


/* turn off the outer cube  */
void turn_off_out_cube(void) {
	PORTA &= ~((1 << PA6) | (1 << PA5));
	PORTC &= ~((1 << PC1) | (1 << PC2));

	line_off(0);
	line_off(3);
}

/* heartbeat model implementation */
void heartbeat(int n)
{
	int i;
	/* no of iterations */
	for (i = 0; i < n; i++) {
		turn_on_in_cube();
		_delay_ms(150);
		turn_off_in_cube();
		turn_on_out_cube();
		_delay_ms(150);
		turn_off_out_cube();
	}
}

/* smiley face model */
void smiley_face(void)
{
	PORTD |= (1<<LAYER1);
	PORTD |= (1<<LAYER2);
	PORTD |= (1<<LAYER3);
	PORTD |= (1<<LAYER4);
	PORTC |= (1 << PC5) | (1 << PC6) ;
	PORTA |= (1 << PA3) | ( 1 << PA6) | (1 << PA5) | ( 1 << PA0);
}

/* lines moving */
void moving_lines(int direction)
{
	int i;
	if (direction == 0) {
		/* left to right */
		for (i = 0; i < 4; i++) {
			line_on(i);
			_delay_ms(100);
			line_off(i);
		}

		for (i = 4; i > 0; i--) {
			line_on(i);
			_delay_ms(100);
			line_off(i);
		}
	}
	else {
		/* up to down */
		for (i = 4; i > 0; i--) {
			layer_on(i);
			_delay_ms(100);
			layer_off(i);
		}
		for (i = 0; i < 4; i++) {
			layer_on(i);
			_delay_ms(100);
			layer_off(i);
		}

	}
}



/* initialize timer */
void init_timer()
{
    /* Fast PWM  with ICR1 and prescaler 1024 */
	TCCR1A |= (1 << WGM11)| (1 << COM1B1)| (1 << COM1A1);
	TCCR1B |= (1 << CS10) | (1 << WGM13) | (1 << WGM12);

    /* ICR1 */
    ICR1 = MHZ/ melody[0];

    OCR1A = ICR1 / 4;
    OCR1B = ICR1 / 2;
}


/* change the model melody depending on the musical frequency */
void melody_model(int model_number)
{
	if (OCR1A < 17000){
			model_type = LAYER2;
		if (model_number == 0) {
			layer_on(1);
		} else if (model_number == 1) {
			line_on(1);
			line_on(2);
		} else {
			turn_on_out_cube();
			outer = TRUE;
		}
	} else if (OCR1A < 13000) {
		if (model_number == 0) {
			model_type = LAYER1;
			layer_on(0);
		} else if (model_number == 1) {
			line_on(0);
		} else {
			turn_on_in_cube();
			inner = TRUE;
		}
 	} else if (OCR1A < 8000) {
		model_type = LAYER4;
		if (model_number == 0) {
			layer_on(3);
		} else if (model_number == 1) {
			line_on(3);
		} else {
			turn_on_in_cube();
			inner = TRUE;
		}
	} else if (OCR1A < 400) {
		model_type = LAYER3;
		if (model_number == 0) {
			layer_on(2);
		} else if (model_number == 1) {
			line_on(2);
		} else {
			turn_on_out_cube();
			outer = TRUE;
		}
	} else {
		light_cube();
		model_type = 5;
	}
}

/* activate buttons */
void activate_buttons()
{
	/* Set PBn pins as input pins. */
	DDRB &= ~(1 << PB0) & ~(1 << PB1);

	/* enable pull up resistance for PB0 and PB1 pins. */
	PORTB |= (1 << PB0) | (1 << PB1);

}

int main(void){
	IO_init();
	IO_music_init();

	activate_buttons();

	int iterations = 0, init = 0, model_number = 0;
	int b_activate = 0;
	int finish_b = 0;
	uint16_t start = 1;

	while(1) 
	{
		/* smiley face mode */
		if (b_activate == 0 &&  finish_b == 0)
				smiley_face();
				
		/* lights models */
		if ((PINB & (1 << PB1)) == 0)
		{
			turn_off_cube();
			rain_effect(16);
			turn_off_cube();
			_delay_ms(100);
			heartbeat(10);
			_delay_ms(100);
			turn_off_cube();
			moving_lines(0);
			turn_off_cube();
			_delay_ms(100);
			moving_lines(1);
			_delay_ms(100);
			moving_lines(0);
			_delay_ms(100);
			moving_lines(1);
		}
		
		/* Mario Lights Game */
		else if ((PINB & (1 << PB0)) == 0) {
			b_activate = 1;
		}

		else if ( b_activate == 1 && finish_b == 0)
		{
			/* buzzer output */
			DDRD |= (1 << PD5);
			turn_off_cube();

			/* initiate interruptions */
			if (init == 0){
				init_timer();
				sei();
				init = 1;
			}

			/* music frequency */
			ICR1 = F_CPU / melody[start];
			OCR1A = ICR1 / 2;
			OCR1B = ICR1 / 4;

			/* melody model */
			melody_model(model_number);
			_delay_ms(current_duration[start] * 1.5);
			start++;

			/* bacstart to start */
			if (start == melody_size) {
				start = 0;
			}
			
			/* change model */
			iterations++;
			if (iterations == 15) {
				model_number = (model_number + 1) % 2;
				iterations = 0;
			}

			/* stop the music */
			if ((PINB & (1 << PB0)) == 0){
				DDRD &= ~(1 << PD5);
				turn_off_cube();
				b_activate = 0;
				finish_b = 1;
			}
			
			/* turn off the cube */
			turn_off_cube();
		}
						
		else 
		{
			/* disables the effect of the cube */
			turn_off_cube();
			smiley_face();
			finish_b = 0;
			b_activate = 0;
		}
	}
	return 0;
}
