#include "mario.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/wdt.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define TRUE 1
#define FALSE 0
#define F_CPU 16000000
#define MHZ 16000000
#define MARIO 0
#define FULL 0xff
#define EMPTY 0x00
#define LAYER1 PD0
#define LAYER2 PD1
#define LAYER3 PD4
#define LAYER4 PD7

/* all 4 layers */
int layers[4] = {LAYER1, LAYER2, LAYER3, LAYER4};

/* layer pins */
int pins[16] = {
	PC3, PC2, PC1, PC0,
	PC7, PC6, PC5, PC4,
	PA3, PA2, PA1, PA0,
	PA7, PA6, PA5, PA4
	};

/* mario info */
int *melody;
int *current_duration;
uint16_t melody_size;
int model_type;

/* for lights models */
int inner, outer;

/* initializations of pins */
void IO_init(void);

/* basic functions for the cube */
void fill_cube(void);
void clear_cube(void);

/* turn on just one layer */
void layer_on(int);
void layer_off(int);

/* turn on just one line of cube */
void line_on(int);
void line_off(int);

/* rain effect */
void drop(int p);
void rain_effect(int iterations);

/* turn on/off in/out cube */
void turn_on_in_cube(void);
void turn_on_out_cube(void);
void turn_off_in_cube(void);
void turn_off_out_cube(void);

/* heartbeat model */
void heartbeat(int n);

/* smiley face model */
void smiley_face(void);

/* lines moving in 2 directions*/
void moving_lines(int direction);

/* initialize timer */
void init_timer();

/* change the model melody depending on the musical frequency */
void melody_model(int model_number);

/* initiate mario song */
void IO_music_init(void);

/* activate buttons */
void activate_buttons();
